<?php
/**
 * Stylite eSync Pro(visioning): policy
 *
 * @link www.stylite.de
 * @author Ralf Becker <rb(at)stylite.de>
 * @copyright (c) 2012 by Ralf Becker <rb(at)stylite.de>
 * @package esyncpro
 * @subpackage policy
 * @version $Id: class.esyncpro_policy.inc.php 2282 2012-07-24 15:15:22Z ralfbecker $
 */

/**
 * Class representing a policy
 */
class esyncpro_policy extends so_sql
{
	const APP = 'esyncpro';
	/**
	 * Name of policy table
	 */
	const TABLE = 'egw_esync_policies';

	/**
	 * Labels for allow-bluetooth values
	 *
	 * @var array
	 */
	static public $allow_bluetooth_lables = array(
		2 => 'Yes',
		1 => 'Only handsfree sets',
		0 => 'No',
	);

	/**
	 * Labels for max-cal-age values
	 *
	 * @var array
	 */
	static public $max_cal_age_labels = array(
		0 => 'All',
		4 => '2 weeks',
		5 => '1 month',
		6 => '3 month',
		7 => '6 month',
	);

	/**
	 * Labels for max-email-age values
	 *
	 * @var array
	 */
	static public $max_email_age_labels = array(
		0 => 'All',
		1 => '1 day',
		2 => '3 days',
		3 => '1 week',
		4 => '2 weeks',
		5 => '1 month',
	);

	/**
	 * Constructor
	 */
	public function __construct()
	{
		parent::__construct(self::APP, self::TABLE, null, '', true);

		$this->convert_all_timestamps();
	}

	/**
	 * saves the content of data to the db
	 *
	 * @param array $keys=null if given $keys are copied to data before saveing => allows a save as
	 * @param string|array $extra_where=null extra where clause, eg. to check an etag, returns true if no affected rows!
	 * @return int|boolean 0 on success, or errno != 0 on error, or true if $extra_where is given and no rows affected
	 */
	function save($keys=null,$extra_where=null)
	{
		if (is_array($keys) && count($keys)) $this->data_merge($keys);

		$this->data['policy_modifier'] = $GLOBALS['egw_info']['user']['account_id'];
		$this->data['policy_modified'] = $this->now;

		return parent::save(null, $extra_where);
	}

	/**
	 * Get policy defaults from tables_current
	 *
	 * @return array
	 */
	static public function defaults()
	{
		static $defaults;

		if (is_null($defaults))
		{
			$defs = $GLOBALS['egw']->db->get_table_definitions(self::APP, self::TABLE);
			$defaults = $defs['fd'];
			foreach($defaults as $name => &$data)
			{
				// as our schema does not use boolean as defaults, but strings '1' and '', we need to cast them to boolean
				if ($data['type'] == 'bool' && isset($data['default']))
				{
					$data['default'] = (boolean)$data['default'];
				}
				$data = $data['default'];
			}
		}
		return $defaults;
	}

	/**
	 * Get standard policy, if non set defaults
	 *
	 * @param boolean $id_only=true false: array with complete standard profile
	 * @return array|int
	 */
	static public function standard($id_only=true)
	{
		$config = config::read(self::APP);

		if ($id_only) return $config['standard_policy'];

		if ($config['standard_policy'])
		{
			$obj = new esyncpro_policy();
			if ($obj->read($config['standard_policy']))
			{
				return $obj->data;
			}
		}
		return self::defaults();
	}

	/**
	 * Set id of standard policy
	 *
	 * @param int $id
	 */
	static public function set_standard($id)
	{
		config::save_value('standard_policy', $id, self::APP);
	}
}