<?php
/**
 * Stylite eSync Pro(visioning): UI
 *
 * @link www.stylite.de
 * @author Ralf Becker <rb(at)stylite.de>
 * @copyright (c) 2012-14 by Ralf Becker <rb(at)stylite.de>
 * @package esyncpro
 * @subpackage policy
 * @version $Id: class.esyncpro_ui.inc.php 3626 2015-03-23 18:17:59Z nathangray $
 */

/**
 * User interface
 */
class esyncpro_ui
{
	/**
	 * Methods callable via menuaction
	 *
	 * @var array
	 */
	public $public_functions = array(
		'policy' => true,
		'device' => true,
		'index'  => true,
		'log'    => true,
	);
	/**
	 * Instance of policy object
	 *
	 * @var esyncpro_policy
	 */
	private static $policy;
	/**
	 * Instance of device object
	 *
	 * @var esyncpro_device
	 */
	private static $device;

	/**
	 * Constructor
	 */
	public function __construct()
	{
		// make sure only admins can call this app
		if (!$GLOBALS['egw_info']['user']['apps']['admin'])
		{
			throw new egw_exception_no_permission_admin();
		}
		if (!isset(self::$policy))
		{
			self::$policy = new esyncpro_policy();
		}
	}

	/**
	 * Fields to keep, if policy_pw_required is NOT checked
	 *
	 * If fields are disabled via jquery, they are not included in $_POST,
	 * but eTemplate has them in to_process and therefore will return them.
	 * This way the values got lost: checkbox are unchecked and integers are 0.
	 *
	 * --> given fields will be restored to old content
	 *
	 * @var array
	 */
	private static $pw_disabled_fields = array(
		'policy_pw_simple',
    	'policy_pw_alphanum',
    	'policy_pw_complexity',
    	'policy_dev_encryption',
    	'policy_card_encryption',
    	'policy_pw_length',
    	'policy_pw_max_failed',
    	'policy_max_inactive_lock',
    	'policy_pw_recovery',
    	'policy_pw_expiration',
    	'policy_pw_history',
	);

	/**
	 * UI policy editor
	 *
	 * @param array $content =null
	 * @param string $msg =''
	 */
	public function policy(array $content=null, $msg='')
	{
		if (!is_array($content))
		{
			if (isset($_GET['policy_id']))
			{
				if (!($content = self::$policy->read($_GET['policy_id'])))
				{
					$msg = lang('Policy not found!');
				}
				else
				{
					$content['standard'] = $content['policy_id'] == esyncpro_policy::standard();
				}
			}
			if (!$content) $content = esyncpro_policy::defaults();
			if(!is_int($content['policy_modified'])) unset($content['policy_modified']);
		}
		else
		{
			if (!$content['policy_pw_required'])	// restore content of disabled fields, see self::$pw_disabled_fields
			{
				$content = array_merge($content, array_intersect_key($content['old'], array_flip(self::$pw_disabled_fields)));
			}
			unset($content['old']);
			//_debug_array($content);
			list($button) = each($content['button']);
			unset($content['button']);
			switch($button)
			{
				case 'save':
				case 'apply':
					foreach(array('policy_max_email_size', 'policy_max_attachment_size') as $name)
					{
						if ((string)$content[$name] === '') $content[$name] = -1;
					}
					self::$policy->init($content);
					if (self::$policy->save() != 0)
					{
						$msg = lang('Error saving policy!');
						unset($button);
					}
					else
					{
						$msg = lang('Policy saved.');
						$content = array_merge($content, self::$policy->data);
					}
					if ($content['standard']) esyncpro_policy::set_standard($content['policy_id']);
					break;
				case 'delete':
					if (self::$policy->delete(array('policy_id' => $content['policy_id'])))
					{
						$msg = lang('Policy deleted.');
					}
					else
					{
						$msg = lang('Error deleting policy!');
						unset($button);
					}
					break;
			}

			egw_framework::refresh_opener($msg, 'admin');
			if (in_array($button, array('save','delete')))
			{
				egw_framework::window_close();
				common::egw_exit();
			}
		}
		$content['msg'] = $msg;
		foreach(array('policy_max_email_size', 'policy_max_attachment_size') as $name)
		{
			if ($content[$name] == -1) $content[$name] = null;
		}
		$readonlys['button[delete]'] = !$content['policy_id'];

		$sel_options = array(
			'policy_allow_bluetooth' => esyncpro_policy::$allow_bluetooth_lables,
			'policy_max_cal_age' => esyncpro_policy::$max_cal_age_labels,
			'policy_max_email_age' => esyncpro_policy::$max_email_age_labels,
		);
		$js .= '$j(function() { toggle_pw_required(); })';
		$GLOBALS['egw_info']['flags']['java_script'] = '<script>'.$js.'</script>';

		$tpl = new etemplate_new('esyncpro.policy');
		$tpl->exec('esyncpro.esyncpro_ui.policy', $content, $sel_options, $readonlys, array(
			'policy_id' => $content['policy_id'],
			'old' => $content,
		), 2);
	}

	/**
	 * UI device editor
	 *
	 * @param array $content =null
	 */
	public function device(array $content=null)
	{
		if (!isset(self::$device)) self::$device = new esyncpro_device();

		$sel_options = $readonlys = $preserv = array();

		$tpl = new etemplate_new('esyncpro.device');
		$tpl->exec('esyncpro.esyncpro_ui.device', $content, $sel_options, $readonlys, $preserv, 2);
	}

	/**
	 * Callback for rows to display
	 *
	 * @param array $query
	 * @param array &$rows
	 * @param array &$readonlys=null
	 * @return int total number of rows
	 */
	public static function get_rows(array $query, array &$rows, array &$readonlys=null)
	{
		if (!isset(self::$device)) self::$device = new esyncpro_device();

		egw_cache::setSession('esyncpro', 'nm_state', $query);

		$query_out = $query;
		if ($query['col_filter']['policy_id'])
		{
			unset($query_out['col_filter']['policy_id']);
			$query_out['col_filter'][] = constant('esyncpro_policy::TABLE').'.policy_id='.$GLOBALS['egw']->db->quote($query['col_filter']['policy_id']);
		}
		if ($query['col_filter']['dev_status'])
		{
			unset($query_out['col_filter']['dev_status']);
			$query_out['col_filter'][] = '('.esyncpro_device::STATUS.')='.$GLOBALS['egw']->db->quote($query['col_filter']['dev_status']);
		}

		return self::$device->get_rows($query_out, $rows, $readonlys, $join=esyncpro_device::STATUS_JOIN,
			false, false, array(esyncpro_device::STATUS.' AS dev_status'));
	}

	/**
	 * UI index page
	 *
	 * @param array $content =null
	 * @param string $msg =''
	 */
	public function index(array $content=null, $msg='')
	{
		if (!is_array($content))
		{
			if (isset($_GET['msg'])) $msg = $_GET['msg'];

			$content['nm'] = egw_cache::getSession('esyncpro', 'nm_state');
			if (!is_array($content['nm']))
			{
				$content['nm'] = array(			// I = value set by the app, 0 = value on return / output
					'get_rows'       =>	__CLASS__.'::get_rows',		// I  method/callback to request the data for the rows eg. 'notes.bo.get_rows'
					//'filter_label'   =>		// I  label for filter    (optional)
					//'filter_help'    =>		// I  help-msg for filter (optional)
					'no_filter'      => True,	// I  disable the 1. filter
					'no_filter2'     => True,	// I  disable the 2. filter (params are the same as for filter)
					'no_cat'         => True,	// I  disable the cat-selectbox
					'start'          =>	0,		// IO position in list
					'order'          =>	'dev_last_sync',	// IO name of the column to sort after (optional for the sortheaders)
					'sort'           =>	'DESC',	// IO direction of the sort: 'ASC' or 'DESC'
					'default_cols'   => '!dev_phoneno,dev_operator,dev_useragent',	// I  columns to use if there's no user or default pref (! as first char uses all but the named columns), default all columns
					'row_id'         => 'dev_id',	// I  key into row content to set it's value as tr id, eg. 'id'
					//'actions'        =>     // I  array with actions, see nextmatch_widget::egw_actions
					//'action_links'   =>     // I  array with enabled actions or ones which should be checked if they are enabled
					//                              optional, default id of all first level actions plus the ones with enabled='javaScript:...'
					'placeholder_actions' => array()
				);
			}
			$content['nm']['actions'] = $this->get_actions();
		}
		elseif($content['nm']['action'])
		{
			$msg = $this->action($content['nm']['action'], $content['nm']['selected']);
			unset($content['nm']['action']);
		}
		$content['msg'] = $msg;

		$sel_options = array(
			'policy_id' => $this->policy_labels(),
			'dev_status' => esyncpro_device::$status_labels,
		);
		$tpl = new etemplate_new('esyncpro.index');
		$tpl->exec('esyncpro.esyncpro_ui.index', $content, $sel_options, array(), $content);
	}

	/**
	 * Get list of available policies
	 *
	 * @param int &$standard=null on return id of standard policy
	 * @return array policy_id => policy_name
	 */
	private function policy_labels(&$standard=null)
	{
		static $policies=null;

		if (!isset($policies))
		{
			$policies = self::$policy->query_list('policy_name', 'policy_id', array(), 'policy_name ASC');

			if (($standard = esyncpro_policy::standard()))
			{
				$policies[$standard] .= ' ('.lang('Standard').')';
			}
		}
		return $policies;
	}

	/**
	 * Get actions / context menu for index
	 *
	 * Changes here, require to log out, as $content['nm'] get stored in session!
	 *
	 * @return array see nextmatch_widget::egw_actions()
	 */
	private function get_actions()
	{
		if (!isset(self::$policy)) self::$policy = new esyncpro_policy();

		$group = 0;
		$actions = array(
			'log' => array(
				'caption' => 'View debug log',
				'icon' => 'view',
				'popup' => '1000x500',
				'url' => 'menuaction=esyncpro.esyncpro_ui.log&log=$id',
				'group' => ++$group,
				'default' => true,
			),
			'policy' => array(
				'caption' => 'Apply policy',
				'children' => $this->policy_labels(),
				'prefix' => 'policy_',
				'group' => $group,
				'icon' => 'apply',
			),
			'wipe' => array(
				'caption' => 'Remote wipe',
				'confirm' => 'Remote wipe this device',
				'confirm_multiple' => 'Remote wipe these devices',
				'group' => ++$group,
				'icon' => 'wipe',
			),
			'delete' => array(
				'caption' => 'Delete',
				'confirm' => 'Delete this device',
				'confirm_multiple' => 'Delete these devices',
				'group' => ++$group,
			),
		);
		//_debug_array($actions);
		return $actions;
	}

	/**
	 * Run given action on selected devices
	 *
	 * @param string $_action
	 * @param array $devices
	 * @return string with message
	 */
	private function action($_action, $devices)
	{
		include_once(EGW_INCLUDE_ROOT.'/activesync/zpushdefs.php');
		if (!isset(self::$device)) self::$device = new esyncpro_device();

		list($action, $action_id) = explode('_', $_action);

		switch($action)
		{
			case 'delete':
				$devices = self::$device->query_list('dev_uid','dev_id',array('dev_id' => $devices));
				$msg = lang('%1 device(s) deleted.', self::$device->delete(array('dev_id' => array_keys($devices))));
				// delete profile directories
				foreach($devices as $dev_uid)
				{
					$path = $GLOBALS['egw_info']['server']['files_dir'].'/activesync/'.strtolower($dev_uid);
					if (!activesync_hooks::rm_recursive($path))
					{
						$msg .= "\n".lang('Deleting of profil %1 failed!', strtolower($dev_uid));
					}
				}
				break;

			case 'wipe':
				$msg = lang('Remote wipe for %1 device(s) requested.',
					self::$device->modify(array(
						'dev_rw_status' => SYNC_PROVISION_RWSTATUS_PENDING,
						'dev_wiped' => null,
					), $devices));
				break;

			case 'policy':
				$msg = lang('Policy of %1 device(s) changed.',
					self::$device->modify(array(
						'policy_id' => $action_id,
						'dev_policy_applied' => null,
					), $devices));
				break;

			default:
				$msg = "Unknown action '$action'!";
		}
		return $msg;
	}

	/**
	 * Hook returning admin menu
	 *
	 * @param string|array $location
	 */
	public static function admin($location)
	{
		if (is_array($location)) $location = $location['location'];

		if (!isset(self::$policy))
		{
			self::$policy = new esyncpro_policy();
		}
		$standard = esyncpro_policy::standard();
		$file = array(
			'Devices' => egw::link('/index.php','menuaction=esyncpro.esyncpro_ui.index'),
		);
		foreach(self::$policy->search('','policy_id,policy_name','policy_name ASC') as $policy)
		{
			$file[] =	array(
				'text'   => $policy['policy_name'].($policy['policy_id'] == $standard ? ' ('.lang('Standard').')' : ''),
				'no_lang' => true,
				'link'    => "javascript:egw_openWindowCentered2('".
					egw::link('/index.php', array(
						'menuaction' => 'esyncpro.esyncpro_ui.policy',
						'policy_id' => $policy['policy_id'],
					), false)."','_blank',750,520,'yes')",
			);
		}
		$file['Add new policy'] = "javascript:egw_openWindowCentered2('".
					egw::link('/index.php', array(
						'menuaction' => 'esyncpro.esyncpro_ui.policy',
					), false)."','_blank',750,520,'yes')";

		$file['View global debug log'] = "javascript:egw_openWindowCentered2('".
					egw::link('/index.php', array(
						'menuaction' => 'esyncpro.esyncpro_ui.log',
						'log' => 'debug.txt',
					), false)."','_blank',1000,500,'yes')";

		$file[] =	array(
			'text' => 'Client support',
			'link' => 'https://en.wikipedia.org/wiki/Comparison_of_Exchange_ActiveSync_clients',
			'target' => '_blank',
		);

		if ($location == 'admin')
		{
			display_section('esyncpro','esyncpro',$file);
		}
		else
		{
			display_sidebox('esyncpro', lang('Available policies'), $file);
		}
	}

	/**
	 * Open log window for log-file specified in GET parameter filename (relative to files_dir)
	 *
	 * $_GET['log'] has to be in activsync sub-dir of files_dir
	 *
	 * @throws egw_exception_wrong_parameter
	 */
	public function log()
	{
		$filename = $_GET['log'];
		if (is_numeric($filename))
		{
			if (!isset(self::$device)) self::$device = new esyncpro_device();
			if (($device = self::$device->read(array('dev_id' => (int)$filename))))
			{
				$filename = strtolower($device['dev_uid']).'/debug.txt';
			}
		}
		if ($filename != 'debug.txt' && !$device)
		{
			throw new egw_exception_wrong_parameter("Access denied to file '$filename'!");
		}
		activesync_hooks::debug_log($filename);
	}
}
